@@ -7,25 +7,25 @@ use crate::utility::prelude::*;
 
 /// Firefly Algorithm settings.
 pub struct Fa {
-    base: BasicSetting,
     alpha: f64,
     beta_min: f64,
     gamma: f64,
 }
 
 impl Fa {
-    impl_builder!(alpha, f64, "Alpha factor.");
-    impl_builder!(beta_min, f64, "Minimum beta factor.");
-    impl_builder!(gamma, f64, "Gamma factor.");
+    impl_builders! {
+        /// Alpha factor.
+        fn alpha(f64)
+        /// Minimum beta factor.
+        fn beta_min(f64)
+        /// Gamma factor.
+        fn gamma(f64)
+    }
 }
 
 impl Default for Fa {
     fn default() -> Self {
         Self {
-            base: BasicSetting {
-                pop_num: 80,
-                ..Default::default()
-            },
             alpha: 0.05,
             beta_min: 0.2,
             gamma: 1.,
@@ -36,15 +36,7 @@ impl Default for Fa {
 impl Setting for Fa {
     type Algorithm = Method;
 
-    fn base(&self) -> &BasicSetting {
-        &self.base
-    }
-
-    fn base_mut(&mut self) -> &mut BasicSetting {
-        &mut self.base
-    }
-
-    fn create(self) -> Self::Algorithm {
+    fn algorithm(self) -> Self::Algorithm {
         Method {
             alpha: self.alpha,
             beta_min: self.beta_min,
@@ -52,6 +44,13 @@ impl Setting for Fa {
             beta0: 1.,
         }
     }
+
+    fn default_basic() -> BasicSetting {
+        BasicSetting {
+            pop_num: 80,
+            ..Default::default()
+        }
+    }
 }
 
 /// Firefly Algorithm type.
@@ -82,8 +81,7 @@ impl Method {
                 }
                 dist
             };
-            #[cfg(feature = "std")]
-            #[allow(unused)]
+            #[cfg(all(feature = "std", not(feature = "libm")))]
             let gamma_r = (-self.gamma * r).exp();
             #[cfg(feature = "libm")]
             let gamma_r = libm::exp(-self.gamma * r);
@@ -91,7 +89,7 @@ impl Method {
             for s in 0..ctx.dim() {
                 let v = ctx.pool[[i, s]]
                     + beta * (pool_j[s] - ctx.pool[[i, s]])
-                    + self.alpha * (ctx.ub(s) - ctx.lb(s)) * rand_float(-0.5, 0.5);
+                    + self.alpha * (ctx.ub(s) - ctx.lb(s)) * ctx.rng.float(-0.5..0.5);
                 tmp[s] = ctx.check(s, v);
             }
             let tmp_f = ctx.func.fitness(tmp.as_slice().unwrap(), &ctx.report);
